@@ -1,4 +1,4 @@
-# $Id: cmon.py 1377 2015-10-27 22:32:30Z mwall $
+# $Id: cmon.py 1547 2016-08-30 00:00:54Z mwall $
 # Copyright 2013 Matthew Wall
 """weewx module that records cpu, memory, disk, and network usage.
 
@@ -126,7 +126,7 @@ from weewx.drivers import AbstractDevice
 from weewx.engine import StdService
 
 DRIVER_NAME = "ComputerMonitor"
-DRIVER_VERSION = "0.13"
+DRIVER_VERSION = "0.14"
 
 if weewx.__version__ < "3":
     raise weewx.UnsupportedFeature("weewx 3 is required, found %s" %
@@ -355,9 +355,13 @@ class LinuxCollector(Collector):
 
         # provide info about the system on which we are running
         loginf('sysinfo: %s' % ' '.join(os.uname()))
-        cpuinfo = self._readproc_dict('/proc/cpuinfo')
-        for key in cpuinfo:
-            loginf('cpuinfo: %s: %s' % (key, cpuinfo[key]))
+        fn = '/proc/cpuinfo'
+        try:
+            cpuinfo = self._readproc_dict(fn)
+            for key in cpuinfo:
+                loginf('cpuinfo: %s: %s' % (key, cpuinfo[key]))
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
 
         self.ignored_mounts = ignored_mounts
         self.last_cpu = dict()
@@ -368,7 +372,7 @@ class LinuxCollector(Collector):
         """read single line proc file, return the string"""
         info = ''
         with open(filename) as fp:
-            info = fp.read()
+            info = fp.readline().strip()
         return info
 
     @staticmethod
@@ -411,84 +415,114 @@ class LinuxCollector(Collector):
         record = super(LinuxCollector, self).get_data(now)
 
         # read memory status
-        meminfo = self._readproc_dict('/proc/meminfo')
-        record['mem_total'] = int(meminfo['MemTotal'].split()[0]) # kB
-        record['mem_free'] = int(meminfo['MemFree'].split()[0]) # kB
-        record['mem_used'] = record['mem_total'] - record['mem_free']
-        record['swap_total'] = int(meminfo['SwapTotal'].split()[0]) # kB
-        record['swap_free'] = int(meminfo['SwapFree'].split()[0]) # kB
-        record['swap_used'] = record['swap_total'] - record['swap_free']
+        fn = '/proc/meminfo'
+        try:
+            meminfo = self._readproc_dict(fn)
+            if meminfo:
+                record['mem_total'] = int(meminfo['MemTotal'].split()[0]) # kB
+                record['mem_free'] = int(meminfo['MemFree'].split()[0]) # kB
+                record['mem_used'] = record['mem_total'] - record['mem_free']
+                record['swap_total'] = int(meminfo['SwapTotal'].split()[0]) # kB
+                record['swap_free'] = int(meminfo['SwapFree'].split()[0]) # kB
+                record['swap_used'] = record['swap_total'] - record['swap_free']
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
 
         # get cpu usage
-        cpuinfo = self._readproc_lines('/proc/stat')
-        values = cpuinfo['cpu'].split()[0:7]
-        for i,key in enumerate(self._CPU_KEYS):
-            if self.last_cpu.has_key(key):
-                record['cpu_'+key] = int(values[i]) - self.last_cpu[key]
-            self.last_cpu[key] = int(values[i])
+        fn = '/proc/stat'
+        try:
+            cpuinfo = self._readproc_lines(fn)
+            if cpuinfo:
+                values = cpuinfo['cpu'].split()[0:7]
+                for i,key in enumerate(self._CPU_KEYS):
+                    if self.last_cpu.has_key(key):
+                        record['cpu_'+key] = int(values[i]) - self.last_cpu[key]
+                    self.last_cpu[key] = int(values[i])
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
 
         # get network usage
-        netinfo = self._readproc_dict('/proc/net/dev')
-        for iface in netinfo:
-            values = netinfo[iface].split()
-            for i, key in enumerate(self._NET_KEYS):
-                if not self.last_net.has_key(iface):
-                    self.last_net[iface] = {}
-                if self.last_net[iface].has_key(key):
-                    record['net_'+iface+'_'+key] = int(values[i]) - self.last_net[iface][key]
-                    if record['net_'+iface+'_'+key] < 0:
-                        maxcnt = 0x100000000 # 32-bit counter
-                        if record['net_'+iface+'_'+key] + maxcnt < 0:
-                            maxcnt = 0x10000000000000000 # 64-bit counter
-                        record['net_'+iface+'_'+key] += maxcnt
-                self.last_net[iface][key] = int(values[i])
+        fn = '/proc/net/dev'
+        try:
+            netinfo = self._readproc_dict(fn)
+            if netinfo:
+                for iface in netinfo:
+                    values = netinfo[iface].split()
+                    for i, key in enumerate(self._NET_KEYS):
+                        if not self.last_net.has_key(iface):
+                            self.last_net[iface] = {}
+                        if self.last_net[iface].has_key(key):
+                            record['net_'+iface+'_'+key] = int(values[i]) - self.last_net[iface][key]
+                            if record['net_'+iface+'_'+key] < 0:
+                                maxcnt = 0x100000000 # 32-bit counter
+                                if record['net_'+iface+'_'+key] + maxcnt < 0:
+                                    maxcnt = 0x10000000000000000 # 64-bit counter
+                                record['net_'+iface+'_'+key] += maxcnt
+                        self.last_net[iface][key] = int(values[i])
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
 
 #        uptimestr = _readproc_line('/proc/uptime')
 #        (uptime,idletime) = uptimestr.split()
 
         # get load and process information
-        loadstr = self._readproc_line('/proc/loadavg')
-        (load1,load5,load15,nproc) = loadstr.split()[0:4]
-        record['load1'] = float(load1)
-        record['load5'] = float(load5)
-        record['load15'] = float(load15)
-
-        (num_proc,tot_proc) = nproc.split('/')
-        record['proc_active'] = int(num_proc)
-        record['proc_total'] = int(tot_proc)
+        fn = '/proc/loadavg'
+        try:
+            loadstr = self._readproc_line(fn)
+            if loadstr:
+                (load1,load5,load15,nproc) = loadstr.split()[0:4]
+                record['load1'] = float(load1)
+                record['load5'] = float(load5)
+                record['load15'] = float(load15)
+                (num_proc,tot_proc) = nproc.split('/')
+                record['proc_active'] = int(num_proc)
+                record['proc_total'] = int(tot_proc)
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
 
         # read cpu temperature
         tdir = '/sys/class/hwmon/hwmon0/device'
         # rpi keeps cpu temperature in a different location
         tfile = '/sys/class/thermal/thermal_zone0/temp'
         if os.path.exists(tdir):
-            for f in os.listdir(tdir):
-                if f.endswith('_input'):
-                    s = self._readproc_line(os.path.join(tdir,f))
-                    if len(s):
-                        n = f.replace('_input','')
-                        tC = int(s) / 1000 # degree C
-                        tF = 32.0 + tC * 9.0 / 5.0
-                        record['cpu_' + n] = tC
+            try:
+                for f in os.listdir(tdir):
+                    if f.endswith('_input'):
+                        s = self._readproc_line(os.path.join(tdir,f))
+                        if s and len(s):
+                            n = f.replace('_input','')
+                            tC = int(s) / 1000 # degree C
+                            tF = 32.0 + tC * 9.0 / 5.0
+                            record['cpu_' + n] = tC
+            except Exception, e:
+                logdbg("read failed for %s: %s" % (tdir, e))
         elif os.path.exists(tfile):
-            s = self._readproc_line(tfile)
-            tC = int(s) / 1000 # degree C
-            record['cpu_temp'] = tC
+            try:
+                s = self._readproc_line(tfile)
+                tC = int(s) / 1000 # degree C
+                record['cpu_temp'] = tC
+            except Exception, e:
+                logdbg("read failed for %s: %s" % (tfile, e))
 
         # get stats on mounted filesystems
+        fn = '/proc/mounts'
         disks = []
-        mntlines = self._readproc_lines('/proc/mounts')
-        for mnt in mntlines:
-            mntpt = mntlines[mnt].split()[0]
-            ignore = False
-            if mnt.find(':') >= 0:
-                ignore = True
-            for m in self.ignored_mounts:
-                if mntpt.startswith(m):
-                    ignore = True
-                    break
-            if not ignore:
-                disks.append(mntpt)
+        try:
+            mntlines = self._readproc_lines(fn)
+            if mntlines:
+                for mnt in mntlines:
+                    mntpt = mntlines[mnt].split()[0]
+                    ignore = False
+                    if mnt.find(':') >= 0:
+                        ignore = True
+                    for m in self.ignored_mounts:
+                        if mntpt.startswith(m):
+                            ignore = True
+                            break
+                    if not ignore:
+                        disks.append(mntpt)
+        except Exception, e:
+            logdbg("read failed for %s: %s" % (fn, e))
         for disk in disks:
             label = disk.replace('/', '_')
             if label == '_':
